<?xml version="1.0"?>
<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format"
    version="1.0">

<xsl:output method="xml" indent="yes"/>

<xsl:param name="font-family-title">Helvetica</xsl:param>
<xsl:param name="font-family-normal">Times Roman</xsl:param>
<xsl:param name="logo">file:/xml/xbe2/chapter6/listings/pslink.jpg</xsl:param>

<xsl:template match="/">
   <fo:root>
      <fo:layout-master-set>
         <fo:simple-page-master master-name="first"
             margin-right="1in" margin-left="1in"
             margin-bottom="1in" margin-top="1in"
             page-width="8.5in" page-height="11in">
            <fo:region-body margin-bottom="0.5in"/>
            <fo:region-after extent="0.2in"/>
         </fo:simple-page-master>
         <fo:simple-page-master master-name="rest"
             margin-right="1in" margin-left="1in"
             margin-bottom="0.5in" margin-top="0.5in"
             page-width="8.5in" page-height="11in">
            <fo:region-before extent="0.5in"/>
            <fo:region-body margin-top="0.5in"
                margin-bottom="0.5in"/>
            <fo:region-after extent="0.2in"/>
         </fo:simple-page-master>
         <fo:page-sequence-master master-name="smart">
            <fo:repeatable-page-master-alternatives>
               <fo:conditional-page-master-reference
                   master-name="first"
                   page-position="first"/>
               <fo:conditional-page-master-reference
                   master-name="rest"/>
            </fo:repeatable-page-master-alternatives>
         </fo:page-sequence-master>
      </fo:layout-master-set>
      <xsl:apply-templates/>
   </fo:root>
</xsl:template>

<xsl:template match="article">
   <fo:page-sequence master-name="smart">
      <fo:static-content flow-name="xsl-region-before">
         <fo:block text-align="end" 
                   font-size="11pt"
                   font-family="{$font-family-title}">
            <xsl:value-of select="articleinfo/title"/>
         </fo:block>
      </fo:static-content>
      <fo:flow flow-name="xsl-region-body">
         <fo:block font-family="{$font-family-normal}"
             font-size="11pt">
            <xsl:apply-templates/>
         </fo:block>
      </fo:flow>
      <fo:static-content flow-name="xsl-region-after">
         <fo:block text-align="end"
                   font-size="11pt"
                   font-family="{$font-family-title}">
            Page <fo:page-number/> of <fo:page-number-citation
                                          ref-id="lastPage"/>
         </fo:block>
      </fo:static-content>
   </fo:page-sequence>
</xsl:template>

<xsl:template match="articleinfo">
   <xsl:apply-templates/>
   <fo:block font-family="{$font-family-title}"
       font-weight="bold" space-after="0.5em">Content</fo:block>
   <fo:table space-after="0.5em">
      <fo:table-column column-width="3in"/>
      <fo:table-column column-width="1in"/>
      <fo:table-body>
         <xsl:for-each select="/article/sect1/title">
            <fo:table-row>
               <fo:table-cell>
                  <fo:block>
                     <fo:basic-link
                         internal-destination="{generate-id()}"
                         color="blue"><xsl:value-of select="."/>
                     </fo:basic-link>
                  </fo:block>
               </fo:table-cell>
               <fo:table-cell>
                  <fo:block text-align="end">
                     <fo:page-number-citation
                         ref-id="{generate-id()}"/>
                  </fo:block>
               </fo:table-cell>
            </fo:table-row>
         </xsl:for-each>
      </fo:table-body>
   </fo:table>
</xsl:template>

<xsl:template match="articleinfo/title">
   <fo:block space-after="1em">
       <fo:external-graphic
           src="{$logo}"/>
   </fo:block>
   <fo:block font-family="{$font-family-title}"
       font-weight="bold" font-size="18pt" space-after="0.5em">
       <xsl:apply-templates/>
   </fo:block>
</xsl:template>

<xsl:template match="author">
   <fo:block font-family="{$font-family-title}"
       font-style="italic" space-after="0.5em">
      <xsl:text>by </xsl:text>
      <xsl:apply-templates/>
   </fo:block>
</xsl:template>

<xsl:template match="sect1/title">
   <fo:block id="{generate-id()}"
      font-family="{$font-family-title}" font-weight="bold"
      space-after="0.5em"><xsl:apply-templates/></fo:block>
</xsl:template>

<xsl:template match="ulink">
   <fo:basic-link external-destination="{@url}"
      color="blue"><xsl:apply-templates/></fo:basic-link>
</xsl:template>

<xsl:template match="emphasis">
   <fo:inline
      font-weight="bold"><xsl:apply-templates/></fo:inline>
</xsl:template>

<xsl:template match="para">
   <fo:block space-after="0.5em">
      <xsl:apply-templates/>
   </fo:block>
</xsl:template>

<xsl:template
    match="sect1[position()=last()]/para[position()=last()]">
   <fo:block>
      <xsl:apply-templates/>
      <fo:inline id="lastPage"
          font-family="ZapfDingbats"> &#167;</fo:inline>
   </fo:block>
</xsl:template>

</xsl:stylesheet>