package com.psol.xbe2;

import java.io.*;
import java.awt.*;
import org.w3c.dom.*;
import java.awt.event.*;
import javax.xml.parsers.*;
import org.apache.xml.serialize.*;

public class ProductsCreator
   extends Frame
{
   protected TextComponent xmlText,
                           name,
                           price;
   protected Choice currencies;
   protected Document document;
   protected static final String[] currencyCode =
   {
      "usd", "eur"
   };
   protected static final String NS =
      "http://www.psol.com/xbe2/listing9.4";

   public ProductsCreator()
      throws ParserConfigurationException
   {
      super("Price List Creator");
      setResizable(false);
      setBackground(Color.lightGray);
      addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent event)
            { System.exit(0); }
      });
      GridBagLayout gridbag = new GridBagLayout();
      GridBagConstraints constraints = new GridBagConstraints();
      setLayout(gridbag);
      xmlText = new TextArea(10,40);
      xmlText.setEditable(false);
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.insets = new Insets(3,3,3,3);
      gridbag.setConstraints(xmlText,constraints);
      add(xmlText);
      Component component = new Label("Name:");
      constraints.gridwidth = 1;
      gridbag.setConstraints(component,constraints);
      add(component);
      name = new TextField(25);
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      gridbag.setConstraints(name,constraints);
      add(name);
      component = new Label("Price:");
      constraints.gridwidth = 1;
      gridbag.setConstraints(component,constraints);
      add(component);
      price = new TextField(15);
      constraints.gridwidth = GridBagConstraints.RELATIVE;
      gridbag.setConstraints(price,constraints);
      add(price);
      currencies = new Choice();
      currencies.add("Dollars");
      currencies.add("Euros");
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      gridbag.setConstraints(currencies,constraints);
      add(currencies);
      Button creator = new Button("Create new product");
      constraints.fill = GridBagConstraints.NONE;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      gridbag.setConstraints(creator,constraints);
      creator.addActionListener(new ActionListener()
      {
         public void actionPerformed(ActionEvent event)
            { doCreate(); }
      });
      add(creator);
      DocumentBuilderFactory factory =
         DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(true);
      DocumentBuilder builder = factory.newDocumentBuilder();
      document = builder.newDocument();
      DOMImplementation implementation =
         document.getImplementation();
      document =
         implementation.createDocument(NS,"xbe:products",null);
      document.getDocumentElement().setAttribute("xmlns:xbe",NS);
   }

   protected void doCreate()
   {
      Element root = document.getDocumentElement(),
              elProduct = document.createElementNS(NS,
                                                  "xbe:product"),
              elName = document.createElementNS(NS,"xbe:name"),
              elPrice = document.createElementNS(NS,"xbe:price");
      String currency =
         currencyCode[currencies.getSelectedIndex()];
      elPrice.setAttribute("currency",currency);
      Text text = document.createTextNode(name.getText());
      elName.appendChild(text);
      text = document.createTextNode(price.getText());
      elPrice.appendChild(text);
      elProduct.appendChild(elName);
      elProduct.appendChild(elPrice);
      root.appendChild(elProduct);
      updateDisplay();
   }

   protected void updateDisplay()
   {
      StringWriter writer = new StringWriter();
      try
      {
         OutputFormat format = new OutputFormat(document);
         format.setIndenting(true);
         XMLSerializer serializer =
            new XMLSerializer(writer,format);
         serializer.asDOMSerializer();
         serializer.serialize(document.getDocumentElement());
      }
      catch(IOException e)
      {
         e.printStackTrace(new PrintWriter(writer));
      }
      xmlText.setText(writer.toString());
   }

   public static void main(String[] params)
      throws ParserConfigurationException
   {
      Frame frame = new ProductsCreator();
      frame.pack();
      frame.show();
   }
}
