package com.psol.xbe2;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.mail.*;
import org.xml.sax.*;
import javax.servlet.*;
import org.apache.soap.*;
import javax.servlet.http.*;
import org.xml.sax.helpers.*;
import javax.xml.transform.*;
import org.apache.soap.rpc.*;
import org.apache.soap.encoding.*;
import org.apache.soap.util.xml.*;
import javax.xml.transform.stream.*;
import org.apache.soap.encoding.soapenc.*;

public class MallServlet
   extends HttpServlet
{
   protected Templates mallTemplates,
                       productTemplates;
   protected SOAPMappingRegistry registry;
   protected URL rpcRouter;

   public void init()
      throws ServletException
   {
      super.init();
      try
      {
         // read the XSLT style sheets
         TransformerFactory factory =
            TransformerFactory.newInstance();
         ClassLoader classLoader = getClass().getClassLoader();
         URL url = classLoader.getResource("etc/mall.xsl");
         Source styleSheet = new StreamSource(url.openStream());
         mallTemplates = factory.newTemplates(styleSheet);
         url = classLoader.getResource("etc/product.xsl");
         styleSheet = new StreamSource(url.openStream());
         productTemplates = factory.newTemplates(styleSheet);
         // prepare the SOAP mapping registry
         BeanSerializer serializer = new BeanSerializer();
         registry = new SOAPMappingRegistry();
         registry.mapTypes(Constants.NS_URI_SOAP_ENC,
                           new QName(
                              "http://www.psol.com/xbe2/chapter12",
                              "Product"),
                           Product.class,serializer,serializer);
         registry.mapTypes(Constants.NS_URI_SOAP_ENC,
                           new QName(
                              "http://www.psol.com/xbe2/chapter12",
                              "ProductDetails"),
                           ProductDetails.class,serializer,serializer);
         registry.mapTypes(Constants.NS_URI_SOAP_ENC,
                           new QName(
                              "http://www.psol.com/xbe2/chapter12",
                              "OrderLine"),
                           OrderLine.class,serializer,serializer);
         // finally get the router's URL
         rpcRouter = new URL(getInitParameter("rpc.router"));
      }
      catch(Exception e)
      {
         throw new UnavailableException(e.getMessage());
      }
   }

   public void doGet(HttpServletRequest request,
                     HttpServletResponse response)
      throws ServletException, IOException
   {
      try
      {
         int productId = getProductId(request);
         if(productId == -1)
            doMall(request,response);
         else
            doProduct(request,response,productId);
      }
      catch(TransformerException e)
      {
         throw new ServletException(e);
      }
      catch(MessagingException e)
      {
         throw new ServletException(e);
      }
      catch(SOAPException e)
      {
         throw new ServletException(e);
      }
   }

   public void doPost(HttpServletRequest request,
                      HttpServletResponse response)
      throws ServletException, IOException
   {
      try
      {
         int productId = getProductId(request);
         if(!isAddressComplete(request))
            doCheckout(request,response,productId);
         else
            doTakeOrder(request,response,productId);
      }
      catch(MessagingException e)
      {
         throw new ServletException(e);
      }
      catch(SOAPException e)
      {
         throw new ServletException(e);
      }
   }

   protected int getProductId(HttpServletRequest request)
   {
      try
      {
         StringTokenizer tokenizer =
            new StringTokenizer(request.getPathInfo(),"/",false);
         String merchantId = tokenizer.nextToken();
         return Integer.parseInt(merchantId);
      }
      catch(Exception e)
      {
         return -1;
      }
   }

   protected void doRpcAndStyle(HttpServletRequest request,
                                HttpServletResponse response,
                                String methodName,
                                Vector params,
                                Templates templates)
      throws IOException, TransformerException,
             MessagingException, SOAPException
   {
      Call call = new Call();
      call.setSOAPMappingRegistry(registry);
      call.setTargetObjectURI("http://www.psol.com/xbe2/chapter12");
      call.setMethodName(methodName);
      call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
      if(params != null)
         call.setParams(params);
      Response resp =
         call.invoke(rpcRouter,
                     "http://www.psol.com/xbe2/soapaction");
      if(!resp.generatedFault())
      {
         Transformer transformer = templates.newTransformer();
         transformer.setParameter("context-path",
                                  request.getContextPath());
         BodyPart bp = resp.getBodyPart(0);
/*         InputStream in = bp.getInputStream();
         for(int c = in.read();c != -1;c = in.read())
            System.out.print((char)c);*/
         Source source = new StreamSource(bp.getInputStream());
         Result result =
            new StreamResult(response.getOutputStream());
         transformer.transform(source,result);
      }
      else
      {
         Fault fault = resp.getFault();
         throw new SOAPException(fault.getFaultCode(),
                                 fault.getFaultString());
      }
   }

   protected void doMall(HttpServletRequest request,
                         HttpServletResponse response)
      throws IOException, TransformerException,
             MessagingException, SOAPException
   {
      doRpcAndStyle(request,response,"getProductList",
                    null,mallTemplates);
   }

   protected void doProduct(HttpServletRequest request,
                            HttpServletResponse response,
                            int productId)
      throws IOException, TransformerException,
             MessagingException, SOAPException
   {
      Vector params = new Vector();
      params.addElement(new Parameter("id",Integer.class,
                                      new Integer(productId),
                                      null));
      doRpcAndStyle(request,response,"getProductDetails",
                    params,productTemplates);
   }

   protected boolean isAddressComplete(HttpServletRequest request)
   {
      // region is not mandatory (only used in the US & Canada)
      String[] fields =
      {
         "name", "street", "postal-code",
         "locality", "country", "email"
      };
      int found = 0;
      for(int i = 0;i < fields.length;i++)
      {
         String value = request.getParameter(fields[i]);
         if(!Util.isEmpty(value))
            found++;
      }
      return found == fields.length;
   }

   protected ProductDetails doGetProductDetails(int productId)
      throws SOAPException
   {
      Call call = new Call();
      call.setSOAPMappingRegistry(registry);
      call.setTargetObjectURI("http://www.psol.com/xbe2/chapter12");
      call.setMethodName("getProductDetails");
      call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
      Vector params = new Vector();
      params.addElement(new Parameter("id",Integer.class,
                                      new Integer(productId),
                                      null));
      call.setParams(params);
      Response resp =
         call.invoke(rpcRouter,
                     "http://www.psol.com/xbe2/soapaction");
      if(!resp.generatedFault())
         return (ProductDetails)resp.getReturnValue().getValue();
      else
      {
         Fault fault = resp.getFault();
         throw new SOAPException(fault.getFaultCode(),
                                 fault.getFaultString());
      }
   }

   protected void writeRow(String label,
                           String key,
                           HttpServletRequest request,
                           Writer writer)
      throws IOException
   {
      writer.write("<tr><td>");
      writer.write(label);
      writer.write("</td><td><input type='text' name='");
      writer.write(key);
      writer.write('\'');
      String value = request.getParameter(key);
      if(!Util.isEmpty(value))
      {
         writer.write(" value='");
         writer.write(value);
         writer.write('\'');
      }
      writer.write("></td></tr>");
   }

   protected void doCheckout(HttpServletRequest request,
                             HttpServletResponse response,
                             int productId)
      throws IOException, SOAPException
   {
      HttpSession session = request.getSession();
      String qt = request.getParameter("quantity");
      if(productId != -1 && qt != null)
      {
         int quantity = Integer.parseInt(qt);
         OrderLine line = (OrderLine)session.getAttribute(
                                      String.valueOf(productId));
         if(line == null)
         {
            ProductDetails product =
               doGetProductDetails(productId);
            line = new OrderLine(product,quantity);
            session.setAttribute(String.valueOf(productId),
                                 line);
         }
         else
            line.setQuantity(quantity);
      }
      PrintWriter writer = response.getWriter();
      writer.write("<html><head><title>Checkout</title></head>");
      writer.write("<body><a href='");
      writer.write(request.getContextPath());
      writer.write("'>Shop some more!</a><form action='");
      writer.write(request.getContextPath());
      writer.write("/' method='post'><table border='0'>");
      writer.write("<tr><td colspan='2'>Please enter your ");
      writer.write("name and address:</td></tr>");
      writeRow("Name *:","name",request,writer);
      writeRow("Company:","company",request,writer);
      writeRow("Street *:","street",request,writer);
      writeRow("Region:","region",request,writer);
      writeRow("ZIP or postal-code *:","postal-code",
               request,writer);
      writeRow("Locality *:","locality",request,writer);
      writeRow("Country *:","country",request,writer);
      writeRow("Email *:","email",request,writer);
      writer.write("</table>");
      writer.write("<table border='0'><tr><td colspan='7'>");
      writer.write("Your order:</td></tr>");
      double total = 0.0;
      Enumeration enum = session.getAttributeNames();
      while(enum.hasMoreElements())
      {
         OrderLine line =
            (OrderLine)session.getAttribute(
                                     (String)enum.nextElement());
         writer.write("<tr><td>");
         writer.write(String.valueOf(line.getQuantity()));
         writer.write("</td><td>*</td><td>");
         writer.write(line.getName());
         writer.write("</td><td>at</td><td>");
         writer.write(String.valueOf(line.getPrice()));
         writer.write("</td><td>=</td><td>");
         writer.write(String.valueOf(line.getTotal()));
         total += line.getTotal();
         writer.write("</td></tr>");
      }
      writer.write("<tr><td colspan='6'>Total:</td><td>");
      writer.write(String.valueOf(total));
      writer.write("</td></tr></table>");
      writer.write("<input type='submit' value='Order'>");
      writer.write("</form></html>");
      writer.flush();
   }

   protected void doTakeOrder(HttpServletRequest request,
                              HttpServletResponse response,
                              int merchantId)
      throws ServletException, IOException,
             MessagingException, SOAPException
   {
      HttpSession session = request.getSession();
      if(session.isNew())
         throw new ServletException("Error with session");
      Enumeration enum = session.getAttributeNames();
      Vector vector = new Vector();
      while(enum.hasMoreElements())
         vector.addElement(session.getAttribute(
                                    (String)enum.nextElement()));
      OrderLine[] lines = new OrderLine[vector.size()];
      vector.copyInto(lines);
      Call call = new Call();
      call.setSOAPMappingRegistry(registry);
      call.setTargetObjectURI("http://www.psol.com/xbe2/chapter12");
      call.setMethodName("addOrder");
      call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
      Vector params = new Vector();
      params.addElement(
         new Parameter("name",String.class,
                       request.getParameter("name"),null));
      params.addElement(
         new Parameter("company",String.class,
                       request.getParameter("company"),
                       null));
      params.addElement(
         new Parameter("street",String.class,
                       request.getParameter("street"),null));
      params.addElement(
         new Parameter("region",String.class,
                       request.getParameter("region"),null));
      params.addElement(
         new Parameter("postalCode",String.class,
                       request.getParameter("postal-code"),null));
      params.addElement(
         new Parameter("locality",String.class,
                       request.getParameter("locality"),null));
      params.addElement(
         new Parameter("country",String.class,
                       request.getParameter("country"),null));
      params.addElement(
         new Parameter("email",String.class,
                       request.getParameter("email"),null));
      params.addElement(
         new Parameter("lines",lines.getClass(),
                       lines,null));
      call.setParams(params);
      /*  Envelope env = call.buildEnvelope();
      Body bp = env.getBody();
      System.out.println(bp.toString()); */
      Response resp =
         call.invoke(rpcRouter,
                     "http://www.psol.com/xbe2/soapaction");
      if(!resp.generatedFault())
      {
         Parameter ret = resp.getReturnValue();
         boolean ok = ((Boolean)ret.getValue()).booleanValue();
         Writer writer = response.getWriter();
         if(ok)
         {
            writer.write("<html><head><title>Checkout</title>");
            writer.write("<body><p>You order has been processed");
            writer.write(" successfully. Thank you for shopping");
            writer.write(" with us.</html>");
         }
         else
         {
            writer.write("<html><head><title>Checkout</title>");
            writer.write("<body><p>Unfortunately you order could");
            writer.write(" not be processed.</html>");
         }
         writer.flush();
         session.invalidate();
      }
      else
      {
         Fault fault = resp.getFault();
         throw new SOAPException(fault.getFaultCode(),
                                 fault.getFaultString());
      }
   }
}
