package com.psol.xbe2;

import java.io.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class MerchantConsole
   extends HttpServlet
{
   protected void doGet(HttpServletRequest request,
                        HttpServletResponse response)
      throws ServletException, IOException
   {
      doProcess(request,response);
   }

   protected void doPost(HttpServletRequest request,
                         HttpServletResponse response)
      throws ServletException, IOException
   {
      doProcess(request,response);
   }

   protected void doProcess(HttpServletRequest request,
                            HttpServletResponse response)
      throws ServletException, IOException
   {
      try
      {
         Class.forName(MerchantService.driver);
         Connection connection =
            DriverManager.getConnection(MerchantService.url,
                                        MerchantService.user,
                                        MerchantService.password);
         try
         {
            String action = request.getParameter("action");
            if(null != action)
            {
               if(action.equalsIgnoreCase("create"))
                  doUpdates(request,connection,createStatements);
               else if(action.equalsIgnoreCase("drop"))
                  doUpdates(request,connection,dropStatements);
               else if(action.equalsIgnoreCase("delete"))
                  doDelete(request,connection);
               else if(action.equalsIgnoreCase("insert"))
                  doInsert(request,connection);
            }
            doPage(request,response,connection);
         }
         finally
         {
            connection.close();
         }
      }
      catch(Exception e)
      {
         e.printStackTrace();
         throw new ServletException(e);
      }
   }

   private static final String[] dropStatements =
   {
      "drop table products",
      "drop table orders",
      "drop table lines",
   };

   private static final String[] createStatements =
   {
      "create table products (id integer not null primary key," +
      "name varchar(50), manufacturer varchar(50), price real," +
      "warranty varchar(20), description varchar(150))",
      "create table orders (id integer not null primary key, " +
      "name varchar(50), company varchar(50), " +
      "street varchar(100), region varchar(50), " +
      "postal_code varchar(15), locality varchar(50), " +
      "country varchar(25), email varchar(50))",
      "create table lines (orderid integer not null, " +
      "productid integer not null, quantity integer not null)"
   };

   protected void doUpdates(HttpServletRequest request,
                            Connection connection,
                            String[] statements)
      throws SQLException
   {
      Statement stmt = connection.createStatement();
      SQLException e = null;
      try
      {
         for(int i = 0;i < statements.length;i++)
            try
               { stmt.executeUpdate(statements[i]); }
            catch(SQLException x)
               { e = e != null ? e : x; }
         if(null != e)
         {
            throw e;
         }
      }
      finally
      {
         stmt.close();
      }
   }

   protected void doDelete(HttpServletRequest request,
                           Connection connection)
      throws SQLException
   {
      PreparedStatement stmt =
         connection.prepareStatement(
            "delete from products where id = ?");
      try
      {
         String id = request.getParameter("id");
         stmt.setInt(1,Integer.parseInt(id));
         stmt.executeUpdate();
      }
      finally
      {
         stmt.close();
      }
   }

   protected void doInsert(HttpServletRequest request,
                           Connection connection)
      throws SQLException, ServletException
   {
      String id = request.getParameter("id"),
             name = request.getParameter("name"),
             manufacturer = request.getParameter("manufacturer"),
             warranty = request.getParameter("warranty"),
             description = request.getParameter("description"),
             price = request.getParameter("price");
      if(Util.isEmpty(id) || Util.isEmpty(name)
         || Util.isEmpty(manufacturer) || Util.isEmpty(warranty)
         || Util.isEmpty(description) || Util.isEmpty(price))
         throw new ServletException("Empty parameter disallowed");
      PreparedStatement stmt =
         connection.prepareStatement(
            "insert into products (id,name,manufacturer," +
            "warranty,description,price) values(?,?,?,?,?,?)");
      try
      {
         stmt.setString(1,id);
         stmt.setString(2,name);
         stmt.setString(3,manufacturer);
         stmt.setString(4,warranty);
         stmt.setString(5,description);
         stmt.setString(6,price);
         stmt.executeUpdate();
      }
      finally
      {
         stmt.close();
      }
   }

   protected boolean isSchemaCreated(Connection connection)
      throws SQLException
   {
      // ask the name of all the tables in the database
      // check if one of them is "products"
      DatabaseMetaData meta = connection.getMetaData();
      ResultSet rs =
         meta.getTables(null,null,null,new String[] { "TABLE" });
      int found = 0;
      while(rs.next())
      {
         String tableName = rs.getString("TABLE_NAME");
         if(tableName.equalsIgnoreCase("products")
            || tableName.equalsIgnoreCase("orders")
            || tableName.equalsIgnoreCase("lines"))
            found++;
      }
      rs.close();
      return 3 == found;
   }

   protected void doPage(HttpServletRequest request,
                         HttpServletResponse response,
                         Connection connection)
      throws SQLException, IOException
   {
      // tell the browser not to cache: otherwise it
      // may be difficult to reload the form
      response.setHeader("Cache-Control","no-cache");
      Writer writer = response.getWriter();
      writer.write("<html><head><title>Merchant Console");
      writer.write("</title></head><body>");
      Statement stmt = connection.createStatement();
      try
      {

         if(isSchemaCreated(connection))
         {
            writer.write("<p><form action='");
            writer.write(request.getRequestURI());
            writer.write("' method='post'><table>");
            writer.write("<tr><td>Identifier:</td>");
            writer.write("<td><input type='text'");
            writer.write(" name='id'></td></tr>");
            writer.write("<tr><td>Name:</td>");
            writer.write("<td><input type='text'");
            writer.write(" name='name'></td></tr>");
            writer.write("<tr><td>Manufacturer:</td>");
            writer.write("<td><input type='text'");
            writer.write(" name='manufacturer'></td></tr>");
            writer.write("<tr><td>Warranty:</td>");
            writer.write("<td><input type='text'");
            writer.write(" name='warranty'></td></tr>");
            writer.write("<tr><td>Description:</td>");
            writer.write("<td><input type='text'");
            writer.write(" name='description'></td></tr>");
            writer.write("<tr><td>Price:</td>");
            writer.write("<td><input type='text'");
            writer.write(" name='price'></td></tr>");
            writer.write("</table><input type='submit'");
            writer.write(" value='Create'>");
            writer.write("<input type='hidden'");
            writer.write(" name='action' value='insert'>");
            writer.write("</form><p>");
            ResultSet rs =
               stmt.executeQuery("select id, name from products");
            writer.write("<table>");
            while(rs.next())
            {
               writer.write("<tr><td>");
               writer.write(rs.getString(2));
               writer.write("</td><td><form action='");
               writer.write(request.getRequestURI());
               writer.write("' method='post'>");
               writer.write(" <input type='submit'");
               writer.write(" value='Delete'>");
               writer.write("<input type='hidden'");
               writer.write(" name='action' value='delete'>");
               writer.write("<input type='HIDDEN'");
               writer.write(" name='id' value='");
               writer.write(rs.getString(1));
               writer.write("'>");
               writer.write("</form></td></tr>");
            }
            writer.write("</table>");
            rs = stmt.executeQuery("select id, name from orders");
            writer.write("<table>");
            while(rs.next())
            {
               int id = rs.getInt(1);
               writer.write("<tr><td valign='top'>");
               writer.write(rs.getString(2));
               writer.write("</td><td>");
               PreparedStatement selectLines =
                  connection.prepareStatement(
                     "select lines.quantity, products.name " +
                     "from lines, products where lines.productid" +
                     " = products.id and lines.orderid = ?");
               try
               {
                  selectLines.setInt(1,id);
                  ResultSet rsl = selectLines.executeQuery();
                  boolean first = true;
                  while(rsl.next())
                  {
                     if(!first)
                        writer.write("<br>");
                     first = false;
                     writer.write(rsl.getString(1));
                     writer.write(" * ");
                     writer.write(rsl.getString(2));
                  }
               }
               finally
               {
                  selectLines.close();
               }
               writer.write("&nbsp;</td></tr>");
            }
            writer.write("</table>");
         }
         writer.write("<p><form action='");
         writer.write(request.getRequestURI());
         writer.write("' method='post'>");
         writer.write("<input type='submit'");
         writer.write(" value='Drop tables'>");
         writer.write("<input type='hidden'");
         writer.write(" name='action' value='drop'>");
         writer.write("</form>");
         writer.write("<form action='");
         writer.write(request.getRequestURI());
         writer.write("' method='post'>");
         writer.write("<input type='submit'");
         writer.write(" value='Create tables'>");
         writer.write("<input type='hidden'");
         writer.write(" name='action' value='create'>");
         writer.write("</form>");
      }
      finally
      {
         stmt.close();
      }
      writer.write("</body></html>");
      writer.flush();
   }
}
